package org.expeditee.core;

import org.expeditee.gui.DisplayController;

/**
 * TODO: Comment. cts16
 * 
 * @author cts16
 */
public class Cursor {

	public enum CursorType {
		DEFAULT,
		TEXT,
		CROSSHAIR,
		CUSTOM
	}
	
	/** Caches the invisible cursor. */
	private static Cursor _invisibleCursor = null;
	
	private Image _image;
	private Point _hotspot;
	private String _name;
	private CursorType _type;
	
	public Cursor(Image image, Point hotspot, String name)
	{
		assert(image != null && hotspot != null && name != null);
		
		_image = image;
		_hotspot = hotspot.clone();
		_name = name;
		_type = CursorType.CUSTOM;
	}
	
	public Cursor(CursorType type)
	{
		_image = null;
		_hotspot = null;
		_name = null;
		_type = type;
	}
	
	public Image getImage()
	{
		return _image;
	}
	
	public Point getHotspot()
	{
		return _hotspot.clone();
	}
	
	public String getName()
	{
		return _name;
	}
	
	public CursorType getType()
	{
		return _type;
	}
	
	/** Creates an invisible cursor. */
	public static Cursor createInvisibleCursor()
	{
		if (_invisibleCursor == null) {
			int[] pixels = new int[DisplayController.SMALL_CURSOR_SIZE * DisplayController.SMALL_CURSOR_SIZE];
			Image image = Image.createImage(DisplayController.SMALL_CURSOR_SIZE, DisplayController.SMALL_CURSOR_SIZE, pixels);
			_invisibleCursor = new Cursor(image, new Point(0, 0), "invisiblecursor");
		}
		
		return _invisibleCursor;
	}
	
}
