package org.expeditee.core;

import org.expeditee.core.bounds.AxisAlignedBoxBounds;
import org.expeditee.core.bounds.Bounds;
import org.expeditee.core.bounds.CombinationBounds;

/**
 * Handles the accumulation of damage areas and clip generation.
 * 
 * @author cts16
 */
public class DamageAreas {

	private CombinationBounds _damagedAreas = null;
	
	public DamageAreas()
	{
		// Do nothing (member allocated on use)
	}
	
	/** Adds an area to the damaged areas. */
	public synchronized void addArea(Bounds area)
	{
		if (_damagedAreas == null) {
			_damagedAreas = new CombinationBounds(area);
		} else {
			_damagedAreas.add(area);
		}
	}
	
	/** Resets the damaged areas to no area damaged. */
	public synchronized void clear()
	{
		_damagedAreas = null;
	}
	
	/**
	 * Returns the rectangular clip enclosing the damaged areas.
	 * Clears the clip at the same time.
	 */
	public synchronized Clip getClip()
	{
		AxisAlignedBoxBounds bounds = AxisAlignedBoxBounds.getEnclosing(_damagedAreas);
		clear();
		if (bounds != null) {
			return new Clip(bounds);
		} else {
			return Clip.getFullClip();
		}
	}
	
}
