package org.expeditee.core;

/**
 * A Dimension represents the 2-dimensional size of something.
 * 
 * @author cts16
 */
public class Dimension {

	/** The size in the x-dimension. */
	public int width;
	/** The size in the y-dimension. */
	public int height;
	
	/** Defaults to a zero-area (degenerate) size if no arguments are given. */
	public Dimension()
	{
		this(0, 0);
	}
	
	/** Standard constructor. */
	public Dimension(int width, int height)
	{
		this.width = width;
		this.height = height;
	}
	
	/** Copy constructor. */
	public Dimension(Dimension other)
	{
		if (other != null) {
			this.width = other.width;
			this.height = other.height;
		} else {
			this.width = 0;
			this.height = 0;
		}
	}
	
	/** Gets the width of this dimension. */
	public int getWidth()
	{
		return width;
	}
	
	/** Gets the height of this dimension. */
	public int getHeight()
	{
		return height;
	}
	
	/** Gets the rectangular area of this dimension. */
	public int getArea()
	{
		return width * height;
	}
	
	/** Sets any negative widths/heights to the absolute equivalents. */
	public void absolve()
	{
		if (width < 0) width = -width;
		if (height <0) height = -height;
	}
	
	@Override
	public boolean equals(Object other)
	{
		if (other instanceof Dimension) {
			Dimension d = (Dimension) other;
			return this.width == d.width && this.height == d.height;
		}
		
		return false;
	}
	
	public Dimension clone()
	{
		return new Dimension(this);
	}
}
