package org.expeditee.core;

/**
 * Represents a fill where the colour is blended from one colour at one point
 * to another colour at another point.
 * 
 * @author cts16
 */
public class GradientFill extends Fill {

	// Fill._colour is treated as the _fromColour (colour at the first point)
	/** The colour at the second point. */
	private Colour _toColour;
	/** The point where the fill should be the first colour. */
	private Point _fromPoint;
	/** The point where the fill should be the second colour. */
	private Point _toPoint;
	/** Whether the gradient repeats at points not between the two specified points. */
	private boolean _acyclic;
	
	/** Standard constructor. */
	public GradientFill(Colour fromColour, Point fromPoint, Colour toColour, Point toPoint, boolean acyclic)
	{
		// Make sure private members are never null
		if (fromColour == null) fromColour = new Colour();
		if (fromPoint == null) fromPoint = new Point();
		if (toColour == null) toColour = new Colour();
		if (toPoint == null) toPoint = new Point();
		
		setFromColour(fromColour);
		setFromPoint(fromPoint);
		setToColour(toColour);
		setToPoint(toPoint);
		setAcyclic(acyclic);
	}
	
	/** If acyclic is unspecified, defaults to true. */
	public GradientFill(Colour fromColour, Point fromPoint, Colour toColour, Point toPoint)
	{
		this(fromColour,  fromPoint, toColour, toPoint, true);
	}
	
	/** Sets the colour to blend away from. */
	public void setFromColour(Colour colour)
	{
		setColour(colour);
	}
	
	/** Gets the colour being blended away from. */
	public Colour getFromColour()
	{
		return getColour();
	}
	
	/** Sets the point to blend away from. */
	public void setFromPoint(Point point)
	{
		_fromPoint = point;
	}

	/** Gets the point being blended away from. */
	public Point getFromPoint()
	{
		return _fromPoint;
	}
	
	/** Sets the colour to blend towards. */
	public void setToColour(Colour colour)
	{
		_toColour = colour;
	}

	/** Gets the colour being blended toward. */
	public Colour getToColour()
	{
		return _toColour;
	}

	/** Sets the point to blend towards. */
	public void setToPoint(Point point)
	{
		_toPoint = point;
	}

	/** Gets the point being blended toward. */
	public Point getToPoint()
	{
		return _toPoint;
	}
	
	/** Sets if this fill should continue blending outside the given two points. */
	public void setAcyclic(boolean acyclic)
	{
		_acyclic = acyclic;
	}

	/** Gets whether this fill continues blending outside the given two points. */
	public boolean isAcyclic()
	{
		return _acyclic;
	}

}
