package org.expeditee.core;

/**
 * Represents how lines/polylines are rendered.
 *  
 * @author cts16
 */
public class Stroke {

	/** The dash array to use for a solid line. */
	public static final float[] SOLID_DASH_ARRAY = new float[] { 1.0f, 0.0f };
	
	/** The methods of rendering the open end of a line. */
	public enum CAP {
		BUTT, // Hard squared-off cap at the logical end of the line
		ROUND, // Semi-circular cap with radius equal to the line thickness
		SQUARE // Similar to BUTT but extends from the logical end of the line by half the line thickness
	}
	
	/** The methods of rendering the joining point between two segments of a polyline. */
	public enum JOIN {
		BEVEL, // Corners of line are connected
		MITER, // Edges of line extend to meet each other
		ROUND // Joins are rendered as circles
	}
	
	/** The thickness of the line. */
	public float thickness;
	/** The method for rendering line ends. */
	public CAP capType;
	/** The method for rendering line joins. */
	public JOIN joinType;
	/** An array of distances for on- and off-strokes when rendering dashed lines. */
	public float[] dashArray;
	/** Where to start in the dash array. */
	public float dashPhase;
	
	/** Default constructor. */
	public Stroke()
	{
		this(1.0f);
	}
	
	/** Standard constructor. Defaults to butt caps, bevel joins and a solid line. */
	public Stroke(float thickness)
	{
		this(thickness, CAP.BUTT, JOIN.BEVEL);
	}
	
	/** Standard constructor. Defaults to a solid line. */
	public Stroke(float thickness, CAP capType, JOIN joinType)
	{
		this(thickness, capType, joinType, SOLID_DASH_ARRAY, 0.0f);
	}
	
	/** Standard constructor. */
	public Stroke(float thickness, CAP capType, JOIN joinType, float[] dashArray, float dashPhase)
	{
		this.thickness = thickness;
		this.capType = capType;
		this.joinType = joinType;
		this.dashArray = dashArray;
		this.dashPhase = dashPhase;
	}
	
	
}
