package org.expeditee.core;

import org.expeditee.core.bounds.AxisAlignedBoxBounds;
import org.expeditee.gio.TextLayoutManager;

/**
 * Purely a handle type for the TextLayoutManager.
 * 
 * @author cts16
 *
 */
public class TextLayout {

	private static TextLayoutManager _manager;
	
	public static void setManager(TextLayoutManager manager)
	{
		if (_manager == null) _manager = manager;
	}
	
	public static TextLayoutManager getManager()
	{
		return _manager;
	}
	
	private static long nextHandle = 1;
	
	private long _handle;
	
	private String _line;
	private Font _font;
	private int _startCharIndex;
	private int _endCharIndex;
	
	public static TextLayout get(String line, Font font, int startCharIndex, int endCharIndex)
	{
		if (_manager == null) return null;
		
		return new TextLayout(line, font, startCharIndex, endCharIndex);
	}
	
	private TextLayout(String line, Font font, int startCharIndex, int endCharIndex)
	{
		_handle = nextHandle;
		nextHandle++;
		
		_line = new String(line);
		_font = new Font(font);
		_startCharIndex = startCharIndex;
		_endCharIndex = endCharIndex;
	}
	
	public long getHandle()
	{
		return _handle;
	}
	
	public float getAdvance()
	{
		return _manager.getAdvance(this);
	}
	
	public int getCharacterCount()
	{
		return _manager.getCharacterCount(this);
	}
	
	public TextHitInfo getNextLeftHit(int offset)
	{
		return _manager.getNextLeftHit(this, offset);
	}
	
	public TextHitInfo getNextLeftHit(TextHitInfo hit)
	{
		return _manager.getNextLeftHit(this, hit);
	}
	
	public TextHitInfo getNextRightHit(int offset)
	{
		return _manager.getNextRightHit(this, offset);
	}
	
	public TextHitInfo getNextRightHit(TextHitInfo hit)
	{
		return _manager.getNextRightHit(this, hit);
	}
	
	public float[] getCaretInfo(TextHitInfo hit)
	{
		return _manager.getCaretInfo(this, hit);
	}
	
	public TextHitInfo hitTestChar(float x, float y)
	{
		return _manager.hitTestChar(this, x, y);
	}
	
	public AxisAlignedBoxBounds getPixelBounds(float x, float y)
	{
		return _manager.getPixelBounds(this, x, y);
	}
	
	public AxisAlignedBoxBounds getLogicalHighlightShape(int firstEndpoint, int secondEndpoint)
	{
		return _manager.getLogicalHighlightShape(this, firstEndpoint, secondEndpoint);
	}
	
	public float getAscent()
	{
		return _manager.getAscent(this);
	}
	
	public float getDescent()
	{
		return _manager.getDescent(this);
	}
	
	public float getLeading()
	{
		return _manager.getLeading(this);
	}
	
	public String getLine()
	{
		return new String(_line);
	}
	
	public Font getFont()
	{
		return new Font(_font);
	}
	
	public int getStartCharIndex()
	{
		return _startCharIndex;
	}
	
	public int getEndCharIndex()
	{
		return _endCharIndex;
	}
	
	public void release()
	{
		_manager.releaseLayout(this);
	}
	
	@Override
	protected void finalize()
	{
		release();
	}
}
