package org.expeditee.gio;

import org.expeditee.core.BlockingRunnable;

/** TODO: Comment. cts16 */
public abstract class MiscManager {

	/**
	 * Causes the machine the interface is running on to make a "beep" sound.
	 */
	public abstract void beep();
	
	/** TODO: Comment. cts16 */
	public abstract boolean browse(String url);
	
	/** TODO: Comment. cts16 */
	public abstract String print(String file);
	
	/** TODO: Comment. cts16 */
	public abstract String open(String file);
	
	/**
	 * Runs the given routine on the dedicated GIO event thread, if the GIO system has one.
	 * 
	 * @param routine
	 * 					Should be a BlockingRunnable if this method should block until
	 * 					the routine has finished, or an ordinary runnable if not.
	 */
	public void runOnGIOThread(Runnable routine)
	{
		// Can't run nothing
		if (routine == null) return;
		
		// If given a blocking runnable, don't return until it has executed
		if (routine instanceof BlockingRunnable) {
			// If we are the GIO thread, can run now
			if (isGIOThread()) {
				routine.run();
				
			// Otherwise schedule and block
			} else {
				scheduleRunnable(routine);
				((BlockingRunnable) routine).blockUntilDone();
			}
			
		// If not a blocking runnable, just schedule the routine for later and return
		} else {
			scheduleRunnable(routine);
		}
	}
	
	/**
	 * Schedules a runnable to be executed at some future time on the GIO thread.
	 */
	protected abstract void scheduleRunnable(Runnable routine);
	
	/**
	 * Returns true if the calling thread is the dedicated GIO thread,
	 * or false if any other thread.
	 */
	protected abstract boolean isGIOThread();
	
}
