package org.expeditee.gio.gesture.data;

import org.expeditee.gui.DisplayController;

public class NavigateFrameGestureData extends GestureData {
	
	public enum NavigateTo {
		ZERO_FRAME, // The zero frame for the current frameset
		FIRST_FRAME, // The one frame for the current frameset
		LAST_FRAME, // The last frame in the current frameset
		NEXT_FRAME, // The (n+1) frame in the current frameset
		PREVIOUS_FRAME, // The (n-1) frame in the current frameset
		FORWARD_FRAME, // Undoes back
		BACK_FRAME, // The previous frame in your history
		EARLIEST_FRAME, // Goes back to the beginning of your history
		LATEST_FRAME, // Goes forward to the end of your history
		SPECIFIC_FRAME // A named frame
	}
	
	private String _currentFrameset;
	private int _currentFrameNumber;
	private NavigateTo _navigateTo;
	
	public NavigateFrameGestureData(NavigateTo navigateTo)
	{
		super();
		_currentFrameset = DisplayController.getCurrentFrame().getFramesetName();
		_currentFrameNumber = DisplayController.getCurrentFrame().getNumber();
		_navigateTo = navigateTo;
	}
	
	public NavigateFrameGestureData(String frameset, int frameNumber)
	{
		super();
		_currentFrameset = frameset;
		_currentFrameNumber = frameNumber;
		_navigateTo = NavigateTo.SPECIFIC_FRAME;
	}
	
	public NavigateFrameGestureData(NavigateFrameGestureData other)
	{
		super(other);
		_currentFrameset = other._currentFrameset;
		_currentFrameNumber = other._currentFrameNumber;
		_navigateTo = other._navigateTo;
		
	}
	
	public String getCurrentFrameset()
	{
		return _currentFrameset;
	}
	
	public int getCurrentFrameNumber()
	{
		return _currentFrameNumber;
	}
	
	public NavigateTo getNavigateTo()
	{
		return _navigateTo;
	}
	
	public NavigateFrameGestureData clone()
	{
		return new NavigateFrameGestureData(this);
	}
}
