package org.expeditee.gio.javafx;

import java.util.Set;

import org.expeditee.core.BlockingRunnable;
import org.expeditee.gio.ClipboardManager;
import org.expeditee.gio.EcosystemManager;

import javafx.scene.input.Clipboard;
import javafx.scene.input.ClipboardContent;
import javafx.scene.input.DataFormat;

public class JavaFXClipboardManager extends ClipboardManager {
	
	/** Data format for identifying Expeditee-specific data on the clipboard. */
	private static final DataFormat expediteeDataFormat = new DataFormat("expediteeDataFormat");
	
	/** Singleton instance. */
	private static JavaFXClipboardManager _instance;
	
	/** Singleton instantiator. */
	public static JavaFXClipboardManager getInstance()
	{
		if (_instance == null) _instance = new JavaFXClipboardManager();
		
		return _instance;
	}
	
	private Clipboard _clipboard;
	
	private JavaFXClipboardManager()
	{
		// Get a reference to the system clipboard
		EcosystemManager.getMiscManager().runOnGIOThread(new BlockingRunnable() {
			@Override
			public void execute()
			{
				_clipboard = Clipboard.getSystemClipboard();
			}
		});
	}

	@Override
	public void set(ClipboardData data)
	{
		// Leave the clipboard unchanged if data is null
		if (data == null) return;
		
		// Fill out the data for the clipboard
		ClipboardContent content = new ClipboardContent();
		if (data.data != null) content.put(expediteeDataFormat, data.data);
		if (data.imageRepresentation != null) {
			javafx.scene.image.Image jfxImage = JavaFXMiscManager.getIfUsingJavaFXImageManager().getInternalImage(data.imageRepresentation);
			content.putImage(jfxImage);
		}
		if (data.stringRepresentation != null) content.putString(data.stringRepresentation);
		
		// Set the clipboard contents
		_clipboard.setContent(content);
		
	}

	@Override
	public ClipboardData get()
	{
		// Create an empty data structure for returning the gathered data
		ClipboardData ret = new ClipboardData();
		
		// See which data formats are available for receipt
		Set<DataFormat> availableFormats = _clipboard.getContentTypes();
		
		// Fill out the return structure
		if (availableFormats.contains(expediteeDataFormat)) ret.data = _clipboard.getContent(expediteeDataFormat);
		if (availableFormats.contains(DataFormat.IMAGE)) {
			javafx.scene.image.Image jfxImage = _clipboard.getImage();
			ret.imageRepresentation = JavaFXMiscManager.getIfUsingJavaFXImageManager().createImage(jfxImage);
		}
		if (availableFormats.contains(DataFormat.PLAIN_TEXT)) ret.stringRepresentation = _clipboard.getString();
		
		// Return it
		return ret;
	}

}
