package org.expeditee.gio.swing;

import java.awt.Desktop;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;

import org.expeditee.Util;
import org.expeditee.core.Image;
import org.expeditee.gio.EcosystemManager;
import org.expeditee.gio.FontManager;
import org.expeditee.gio.GraphicsManager;
import org.expeditee.gio.ImageManager;
import org.expeditee.gio.MiscManager;
import org.expeditee.gio.TextLayoutManager;

import javafx.embed.swing.SwingFXUtils;

public class SwingMiscManager extends MiscManager {
	
	private static SwingMiscManager _instance;
	
	public static SwingMiscManager getInstance()
	{
		if (_instance == null) _instance = new SwingMiscManager();
		
		return _instance;
	}
	
	public static void setJButtonIcon(AbstractButton jbutton, Image image)
	{
		if (jbutton == null || image == null) return;
		
		SwingImageManager manager = getIfUsingSwingImageManager();
		
		if (manager == null) return;
		
		Icon icon = new ImageIcon(manager.getInternalImage(image));
		
		jbutton.setIcon(icon);
	}
	
	public static void setJButtonSelectedIcon(AbstractButton jbutton, Image image)
	{
		if (jbutton == null || image == null) return;
		
		SwingImageManager manager = getIfUsingSwingImageManager();
		
		if (manager == null) return;
		
		Icon icon = new ImageIcon(manager.getInternalImage(image));
		
		jbutton.setSelectedIcon(icon);
	}
	
	/**
	 * Returns the SwingImageManager if it is the ImageManager for this session,
	 * or null if another ImageManager is being used.
	 */
	public static SwingImageManager getIfUsingSwingImageManager()
	{
		ImageManager manager = org.expeditee.core.Image.getManager();
		if (manager instanceof SwingImageManager) {
			return (SwingImageManager) manager;
		}
		
		return null;
	}
	
	/**
	 * Returns the SwingFontManager if it is the FontManager for this session,
	 * or null if another FontManager is being used.
	 */
	public static SwingFontManager getIfUsingSwingFontManager()
	{
		FontManager manager = EcosystemManager.getFontManager();
		if (manager instanceof SwingFontManager) {
			return (SwingFontManager) manager;
		}
		
		return null;
	}
	
	/**
	 * Returns the SwingGraphicsManager if it is the GraphicsManager for this session,
	 * or null if another GraphicsManager is being used.
	 */
	public static SwingGraphicsManager getIfUsingSwingGraphicsManager()
	{
		GraphicsManager manager = EcosystemManager.getGraphicsManager();
		if (manager instanceof SwingGraphicsManager) {
			return (SwingGraphicsManager) manager;
		}
		
		return null;
	}
	
	/**
	 * Returns the SwingTextLayoutManager if it is the TextLayoutManager for this session,
	 * or null if another TextLayoutManager is being used.
	 */
	public static SwingTextLayoutManager getIfUsingSwingTextLayoutManager()
	{
		TextLayoutManager manager = EcosystemManager.getTextLayoutManager();
		if (manager instanceof SwingTextLayoutManager) {
			return (SwingTextLayoutManager) manager;
		}
		
		return null;
	}
	
	public static org.expeditee.core.Image getImageForJavaFXImage(javafx.scene.image.Image image)
	{
		BufferedImage swingImage = SwingFXUtils.fromFXImage(image, null);
		
		return getIfUsingSwingImageManager().createImage(swingImage);
	}
	
	@Override
	public void beep()
	{
		java.awt.Toolkit.getDefaultToolkit().beep();
	}
	
	@Override
	public boolean browse(String url)
	{
		try {
			URL check = new URL(url); // TODO: Why do we need this? cts16	
			Desktop.getDesktop().browse(new URI(url));
		} catch (IOException | URISyntaxException e) {
			return false;
		}
		
		return true;
	}
	
	@Override
	public String print(String file)
	{
		try {
			if (Util.isMinimumVersion6()) {
				if (Desktop.isDesktopSupported()) {
					Desktop.getDesktop().print(new File(file));
				}
			}
		} catch (Exception e) {
			return e.getMessage();
		}
		
		return null;
	}

	@Override
	public String open(String file)
	{
		try {
			if (Util.isMinimumVersion6()) {
				if (Desktop.isDesktopSupported()) {
					Desktop.getDesktop().open(new File(file));
				}
			}
		} catch (Exception e) {
			return e.getStackTrace().toString();
		}
		
		return null;
	}

	@Override
	protected void scheduleRunnable(Runnable routine)
	{
		SwingUtilities.invokeLater(routine);
	}

	@Override
	protected boolean isGIOThread()
	{
		return SwingUtilities.isEventDispatchThread();
	}
}
